@@ -1,17 +1,10 @@
-# -*- coding: utf-8 -*-
 # Copyright: See the LICENSE file.
 
-from __future__ import unicode_literals
 
 import collections
 import logging
-import warnings
 
-from . import builder
-from . import declarations
-from . import enums
-from . import errors
-from . import utils
+from . import builder, declarations, enums, errors, utils
 
 logger = logging.getLogger('factory.generate')
 
@@ -47,7 +40,7 @@ class FactoryMetaClass(type):
         elif cls._meta.strategy == enums.STUB_STRATEGY:
             return cls.stub(**kwargs)
         else:
-            raise errors.UnknownStrategy('Unknown Meta.strategy: {0}'.format(
+            raise errors.UnknownStrategy('Unknown Meta.strategy: {}'.format(
                 cls._meta.strategy))
 
     def __new__(mcs, class_name, bases, attrs):
@@ -80,7 +73,7 @@ class FactoryMetaClass(type):
         meta = options_class()
         attrs['_meta'] = meta
 
-        new_class = super(FactoryMetaClass, mcs).__new__(
+        new_class = super().__new__(
             mcs, class_name, bases, attrs)
 
         meta.contribute_to_class(
@@ -105,7 +98,7 @@ class BaseMeta:
     strategy = enums.CREATE_STRATEGY
 
 
-class OptionDefault(object):
+class OptionDefault:
     """The default for an option.
 
     Attributes:
@@ -140,7 +133,7 @@ class OptionDefault(object):
             self.name, self.value, self.inherit)
 
 
-class FactoryOptions(object):
+class FactoryOptions:
     def __init__(self):
         self.factory = None
         self.base_factory = None
@@ -166,8 +159,16 @@ class FactoryOptions(object):
         Custom FactoryOptions classes should override this method
         to update() its return value.
         """
+
+        def is_model(meta, value):
+            if isinstance(value, FactoryMetaClass):
+                raise TypeError(
+                    "%s is already a %s"
+                    % (repr(value), Factory.__name__)
+                )
+
         return [
-            OptionDefault('model', None, inherit=True),
+            OptionDefault('model', None, inherit=True, checker=is_model),
             OptionDefault('abstract', False, inherit=False),
             OptionDefault('strategy', enums.CREATE_STRATEGY, inherit=True),
             OptionDefault('inline_args', (), inherit=True),
@@ -180,11 +181,11 @@ class FactoryOptions(object):
         if meta is None:
             meta_attrs = {}
         else:
-            meta_attrs = dict(
-                (k, v)
+            meta_attrs = {
+                k: v
                 for (k, v) in vars(meta).items()
                 if not k.startswith('_')
-            )
+            }
 
         for option in self._build_default_options():
             assert not hasattr(self, option.name), "Can't override field %s." % option.name
@@ -374,7 +375,7 @@ class FactoryOptions(object):
         return self.model
 
     def __str__(self):
-        return "<%s for %s>" % (self.__class__.__name__, self.factory.__class__.__name__)
+        return "<%s for %s>" % (self.__class__.__name__, self.factory.__name__)
 
     def __repr__(self):
         return str(self)
@@ -383,7 +384,7 @@ class FactoryOptions(object):
 # Factory base classes
 
 
-class _Counter(object):
+class _Counter:
     """Simple, naive counter.
 
     Attributes:
@@ -403,7 +404,7 @@ class _Counter(object):
         self.seq = next_value
 
 
-class BaseFactory(object):
+class BaseFactory:
     """Factory base support for sequences, attributes and stubs."""
 
     # Backwards compatibility
@@ -440,44 +441,6 @@ class BaseFactory(object):
         """
         return 0
 
-    @classmethod
-    def attributes(cls, create=False, extra=None):
-        """Build a dict of attribute values, respecting declaration order.
-
-        The process is:
-        - Handle 'orderless' attributes, overriding defaults with provided
-            kwargs when applicable
-        - Handle ordered attributes, overriding them with provided kwargs when
-            applicable; the current list of computed attributes is available
-            to the currently processed object.
-        """
-        warnings.warn(
-            "Usage of Factory.attributes() is deprecated.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        declarations = cls._meta.pre_declarations.as_dict()
-        declarations.update(extra or {})
-        from . import helpers
-        return helpers.make_factory(dict, **declarations)
-
-    @classmethod
-    def declarations(cls, extra_defs=None):
-        """Retrieve a copy of the declared attributes.
-
-        Args:
-            extra_defs (dict): additional definitions to insert into the
-                retrieved DeclarationDict.
-        """
-        warnings.warn(
-            "Factory.declarations is deprecated; use Factory._meta.pre_declarations instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        decls = cls._meta.pre_declarations.as_dict()
-        decls.update(extra_defs or {})
-        return decls
-
     @classmethod
     def _adjust_kwargs(cls, **kwargs):
         """Extension point for custom kwargs adjustment."""
@@ -663,23 +626,22 @@ class BaseFactory(object):
         return cls.generate_batch(strategy, size, **kwargs)
 
 
-# Note: we're calling str() on the class name to avoid issues with Py2's type() expecting bytes
-# instead of unicode.
-Factory = FactoryMetaClass(str('Factory'), (BaseFactory,), {
-    'Meta': BaseMeta,
-    '__doc__': """Factory base with build and create support.
+class Factory(BaseFactory, metaclass=FactoryMetaClass):
+    """Factory base with build and create support.
 
     This class has the ability to support multiple ORMs by using custom creation
     functions.
-    """,
-})
+    """
+
+    class Meta(BaseMeta):
+        pass
 
 
 # Backwards compatibility
 Factory.AssociatedClassError = errors.AssociatedClassError
 
 
-class StubObject(object):
+class StubObject:
     """A generic container."""
     def __init__(self, **kwargs):
         for field, value in kwargs.items():
